@@ -1,19 +1,7 @@
 # Copyright (C) 2013, 2014 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import os
 import unittest
@@ -24,30 +12,12 @@ from virtinst import Capabilities
 from virtinst import DomainCapabilities
 
 
-conn = utils.open_testdriver()
-
-
 class TestCapabilities(unittest.TestCase):
     def _buildCaps(self, filename):
         path = os.path.join("tests/capabilities-xml", filename)
+        conn = utils.URIs.open_testdefault_cached()
         return Capabilities(conn, open(path).read())
 
-    def testCapsCPUFeaturesOldSyntax(self):
-        filename = "test-old-vmx.xml"
-        host_feature_list = ["vmx"]
-
-        caps = self._buildCaps(filename)
-        for f in host_feature_list:
-            self.assertEqual(caps.host.cpu.has_feature(f), True)
-
-    def testCapsCPUFeaturesOldSyntaxSVM(self):
-        filename = "test-old-svm.xml"
-        host_feature_list = ["svm"]
-
-        caps = self._buildCaps(filename)
-        for f in host_feature_list:
-            self.assertEqual(caps.host.cpu.has_feature(f), True)
-
     def testCapsCPUFeaturesNewSyntax(self):
         filename = "test-qemu-with-kvm.xml"
         host_feature_list = ['lahf_lm', 'xtpr', 'cx16', 'tm2', 'est', 'vmx',
@@ -55,7 +25,8 @@ class TestCapabilities(unittest.TestCase):
 
         caps = self._buildCaps(filename)
         for f in host_feature_list:
-            self.assertEqual(caps.host.cpu.has_feature(f), True)
+            self.assertEqual(
+                    f in [feat.name for feat in caps.host.cpu.features], True)
 
         self.assertEqual(caps.host.cpu.model, "core2duo")
         self.assertEqual(caps.host.cpu.vendor, "Intel")
@@ -66,7 +37,7 @@ class TestCapabilities(unittest.TestCase):
     def testCapsUtilFuncs(self):
         caps_with_kvm = self._buildCaps("test-qemu-with-kvm.xml")
         caps_no_kvm = self._buildCaps("test-qemu-no-kvm.xml")
-        caps_empty = self._buildCaps("test-old-vmx.xml")
+        caps_empty = self._buildCaps("test-empty.xml")
 
         def test_utils(caps, has_guests, is_kvm):
             if caps.guests:
@@ -122,7 +93,7 @@ class TestCapabilities(unittest.TestCase):
 
     def testDomainCapabilities(self):
         xml = open("tests/capabilities-xml/test-domcaps.xml").read()
-        caps = DomainCapabilities(utils.open_testdriver(), xml)
+        caps = DomainCapabilities(utils.URIs.open_testdriver_cached(), xml)
 
         self.assertEqual(caps.os.loader.supported, True)
         self.assertEqual(caps.os.loader.get_values(),
@@ -131,6 +102,17 @@ class TestCapabilities(unittest.TestCase):
         self.assertEqual(caps.os.loader.get_enum("type").get_values(),
             ["rom", "pflash"])
 
+    def testDomainCapabilitiesx86(self):
+        xml = open("tests/capabilities-xml/kvm-x86_64-domcaps.xml").read()
+        caps = DomainCapabilities(utils.URIs.open_testdriver_cached(), xml)
 
-if __name__ == "__main__":
-    unittest.main()
+        self.assertEqual(caps.machine, "pc-i440fx-2.1")
+        self.assertEqual(caps.arch, "x86_64")
+        self.assertEqual(caps.domain, "kvm")
+        self.assertEqual(caps.path, "/bin/qemu-system-x86_64")
+
+        custom_mode = caps.cpu.get_mode("custom")
+        self.assertTrue(bool(custom_mode))
+        cpu_model = custom_mode.get_model("Opteron_G4")
+        self.assertTrue(bool(cpu_model))
+        self.assertTrue(cpu_model.usable)
