@@ -1,19 +1,7 @@
 # Copyright (C) 2017 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import os
 import unittest
@@ -52,3 +40,11 @@ class TestHostkeymap(unittest.TestCase):
                 hostkeymap._xorg_keymap(
                     _open("xorg-rhel5.txt")),
                 "us")
+
+    def testSanitize(self):
+        def san(inp):
+            return hostkeymap.sanitize_keymap(inp)
+
+        self.assertEqual(san("pt"), "pt")
+        self.assertEqual(san("de-us"), "de")
+        self.assertEqual(san("en_us"), "en-us")
