@@ -1,22 +1,8 @@
-#
 # Copyright (C) 2009, 2013 Red Hat, Inc.
 # Copyright (C) 2009 Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 import logging
 import traceback
@@ -28,8 +14,9 @@ from gi.repository import Pango
 from virtinst import util
 
 from . import uiutil
-from .baseclass import vmmGObjectUI
 from .asyncjob import vmmAsyncJob
+from .baseclass import vmmGObjectUI
+from .connmanager import vmmConnectionManager
 from .domain import vmmDomain
 
 
@@ -40,31 +27,46 @@ NUM_COLS = 3
 
 
 class vmmMigrateDialog(vmmGObjectUI):
-    def __init__(self, engine):
+    @classmethod
+    def show_instance(cls, parentobj, vm):
+        try:
+            if not cls._instance:
+                cls._instance = vmmMigrateDialog()
+            cls._instance.show(parentobj.topwin, vm)
+        except Exception as e:
+            parentobj.err.show_err(
+                    _("Error launching migrate dialog: %s") % str(e))
+
+    def __init__(self):
         vmmGObjectUI.__init__(self, "migrate.ui", "vmm-migrate")
         self.vm = None
-        self.conn = None
-        self._conns = {}
 
         self.builder.connect_signals({
-            "on_vmm_migrate_delete_event" : self._delete_event,
-            "on_migrate_cancel_clicked" : self._cancel_clicked,
-            "on_migrate_finish_clicked" : self._finish_clicked,
+            "on_vmm_migrate_delete_event": self._delete_event,
+            "on_migrate_cancel_clicked": self._cancel_clicked,
+            "on_migrate_finish_clicked": self._finish_clicked,
 
-            "on_migrate_dest_changed" : self._destconn_changed,
-            "on_migrate_set_address_toggled" : self._set_address_toggled,
-            "on_migrate_set_port_toggled" : self._set_port_toggled,
-            "on_migrate_mode_changed" : self._mode_changed,
+            "on_migrate_dest_changed": self._destconn_changed,
+            "on_migrate_set_address_toggled": self._set_address_toggled,
+            "on_migrate_set_port_toggled": self._set_port_toggled,
+            "on_migrate_mode_changed": self._mode_changed,
         })
         self.bind_escape_key_close()
+        self._cleanup_on_app_close()
 
-        self._init_state(engine)
+        self._init_state()
 
 
     def _cleanup(self):
         self.vm = None
-        self.conn = None
-        self._conns = None
+
+    @property
+    def _connobjs(self):
+        return vmmConnectionManager.get_instance().conns
+
+    @property
+    def conn(self):
+        return self.vm and self.vm.conn or None
 
 
     ##############
@@ -73,8 +75,7 @@ class vmmMigrateDialog(vmmGObjectUI):
 
     def show(self, parent, vm):
         logging.debug("Showing migrate wizard")
-        self.vm = vm
-        self.conn = vm.conn
+        self._set_vm(vm)
         self._reset_state()
         self.topwin.set_transient_for(parent)
         self.topwin.present()
@@ -82,14 +83,27 @@ class vmmMigrateDialog(vmmGObjectUI):
     def close(self, ignore1=None, ignore2=None):
         logging.debug("Closing migrate wizard")
         self.topwin.hide()
+        self._set_vm(None)
         return 1
 
+    def _vm_removed(self, _conn, connkey):
+        if self.vm.get_connkey() == connkey:
+            self.close()
+
+    def _set_vm(self, newvm):
+        oldvm = self.vm
+        if oldvm:
+            oldvm.conn.disconnect_by_obj(self)
+        if newvm:
+            newvm.conn.connect("vm-removed", self._vm_removed)
+        self.vm = newvm
+
 
     ################
     # Init helpers #
     ################
 
-    def _init_state(self, engine):
+    def _init_state(self):
         blue = Gdk.color_parse("#0072A8")
         self.widget("header").modify_bg(Gtk.StateType.NORMAL, blue)
 
@@ -108,13 +122,16 @@ class vmmMigrateDialog(vmmGObjectUI):
         model.set_sort_column_id(COL_LABEL, Gtk.SortType.ASCENDING)
 
         def _sorter(model, iter1, iter2, ignore):
+            def _cmp(a, b):
+                return ((a > b) - (a < b))
+
             row1 = model[iter1]
             row2 = model[iter2]
             if row1[COL_URI] is None:
                 return -1
             if row2[COL_URI] is None:
                 return 1
-            return cmp(row1[COL_LABEL], row2[COL_LABEL])
+            return _cmp(row1[COL_LABEL], row2[COL_LABEL])
         model.set_sort_func(COL_LABEL, _sorter)
 
         # Mode combo
@@ -126,9 +143,6 @@ class vmmMigrateDialog(vmmGObjectUI):
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 0)
 
-        # Hook up signals to get connection listing
-        engine.connect("conn-added", self._conn_added_cb)
-        engine.connect("conn-removed", self._conn_removed_cb)
         self.widget("migrate-dest").emit("changed")
 
         self.widget("migrate-mode").set_tooltip_text(
@@ -190,7 +204,7 @@ class vmmMigrateDialog(vmmGObjectUI):
         self._finish()
 
     def _destconn_changed(self, src):
-        row = uiutil.get_list_selection(src, None)
+        row = uiutil.get_list_selected_row(src)
         if not row:
             return
 
@@ -210,8 +224,8 @@ class vmmMigrateDialog(vmmGObjectUI):
         tunnel_warning = ""
         tunnel_uri = ""
 
-        if can_migrate and uri in self._conns:
-            destconn = self._conns[uri]
+        if can_migrate and uri in self._connobjs:
+            destconn = self._connobjs[uri]
 
             tunnel_uri = destconn.get_uri()
             if not destconn.is_remote():
@@ -258,7 +272,7 @@ class vmmMigrateDialog(vmmGObjectUI):
         self.widget("migrate-port-label").set_visible(not enable)
 
     def _is_tunnel_selected(self):
-        return uiutil.get_list_selection(self.widget("migrate-mode"), 1)
+        return uiutil.get_list_selection(self.widget("migrate-mode"), column=1)
 
     def _mode_changed(self, src):
         ignore = src
@@ -266,14 +280,6 @@ class vmmMigrateDialog(vmmGObjectUI):
         self.widget("migrate-direct-box").set_visible(not is_tunnel)
         self.widget("migrate-tunnel-box").set_visible(is_tunnel)
 
-    def _conn_added_cb(self, engine, conn):
-        ignore = engine
-        self._conns[conn.get_uri()] = conn
-
-    def _conn_removed_cb(self, engine, uri):
-        ignore = engine
-        del(self._conns[uri])
-
 
     ###########################
     # destconn combo handling #
@@ -310,7 +316,7 @@ class vmmMigrateDialog(vmmGObjectUI):
         model.clear()
 
         rows = []
-        for conn in self._conns.values():
+        for conn in list(self._connobjs.values()):
             rows.append(self._build_dest_row(conn))
 
         if not any([row[COL_CAN_MIGRATE] for row in rows]):
@@ -344,7 +350,7 @@ class vmmMigrateDialog(vmmGObjectUI):
             return
 
         if self.conn.is_xen():
-            uri = "xenmigr://%s" % address
+            uri = "%s" % address
         else:
             uri = "tcp:%s" % address
         if port:
@@ -352,23 +358,21 @@ class vmmMigrateDialog(vmmGObjectUI):
         return uri
 
     def _finish_cb(self, error, details, destconn):
-        self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.reset_finish_cursor()
 
         if error:
             error = _("Unable to migrate guest: %s") % error
             self.err.show_err(error, details=details)
         else:
-            self.conn.schedule_priority_tick(pollvm=True)
             destconn.schedule_priority_tick(pollvm=True)
+            self.conn.schedule_priority_tick(pollvm=True)
             self.close()
 
     def _finish(self):
         try:
-            row = uiutil.get_list_selection(self.widget("migrate-dest"), None)
+            row = uiutil.get_list_selected_row(self.widget("migrate-dest"))
             destlabel = row[COL_LABEL]
-            destconn = self._conns.get(row[COL_URI])
+            destconn = self._connobjs.get(row[COL_URI])
 
             tunnel = self._is_tunnel_selected()
             unsafe = self.widget("migrate-unsafe").get_active()
@@ -378,16 +382,14 @@ class vmmMigrateDialog(vmmGObjectUI):
                 uri = self.widget("migrate-tunnel-uri").get_text()
             else:
                 uri = self._build_regular_migrate_uri()
-        except Exception, e:
+        except Exception as e:
             details = "".join(traceback.format_exc())
             self.err.show_err((_("Uncaught error validating input: %s") %
                                str(e)),
                                details=details)
             return
 
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(
-            Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        self.set_finish_cursor()
 
         cancel_cb = None
         if self.vm.getjobinfo_supported:
@@ -400,7 +402,7 @@ class vmmMigrateDialog(vmmGObjectUI):
             self._async_migrate,
             [self.vm, destconn, uri, tunnel, unsafe, temporary],
             self._finish_cb, [destconn],
-            _("Migrating VM '%s'" % self.vm.get_name()),
+            _("Migrating VM '%s'") % self.vm.get_name(),
             (_("Migrating VM '%s' to %s. This may take a while.") %
              (self.vm.get_name(), destlabel)),
             self.topwin, cancel_cb=cancel_cb)
@@ -413,7 +415,7 @@ class vmmMigrateDialog(vmmGObjectUI):
 
         try:
             vm.abort_job()
-        except Exception, e:
+        except Exception as e:
             logging.exception("Error cancelling migrate job")
             asyncjob.show_warning(_("Error cancelling migrate job: %s") % e)
             return
