@@ -1,29 +1,16 @@
-#
 # Copyright (C) 2014, 2015 Red Hat, Inc.
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
+import functools
 import logging
 import os
-import Queue
+import queue
 import socket
 import signal
 import threading
-import ipaddr
+import ipaddress
 
 from .baseclass import vmmGObject
 
@@ -33,11 +20,12 @@ class ConnectionInfo(object):
     Holds all the bits needed to make a connection to a graphical console
     """
     def __init__(self, conn, gdev):
-        self.gtype      = gdev.type
-        self.gport      = gdev.port and str(gdev.port) or None
-        self.gsocket    = gdev.socket
-        self.gaddr      = gdev.listen or "127.0.0.1"
-        self.gtlsport   = gdev.tlsPort or None
+        self.gtype = gdev.type
+        self.gport = gdev.port and str(gdev.port) or None
+        self.gsocket = (gdev.listens and gdev.listens[0].socket) or gdev.socket
+        self.gaddr = gdev.listen or "127.0.0.1"
+        self.gtlsport = gdev.tlsPort or None
+        self.glistentype = gdev.get_first_listen_type()
 
         self.transport = conn.get_uri_transport()
         self.connuser = conn.get_uri_username()
@@ -49,56 +37,67 @@ class ConnectionInfo(object):
 
     def _is_listen_localhost(self, host=None):
         try:
-            return ipaddr.IPNetwork(host or self.gaddr).is_loopback
-        except:
+            return ipaddress.ip_network(str(host or self.gaddr)).is_loopback
+        except Exception:
             return False
 
     def _is_listen_any(self):
         try:
-            return ipaddr.IPNetwork(self.gaddr).is_unspecified
-        except:
+            return ipaddress.ip_network(str(self.gaddr)).is_unspecified
+        except Exception:
             return False
 
+    def _is_listen_none(self):
+        if self.glistentype == "none":
+            return True
+        return not (self.gsocket or self.gport or self.gtlsport)
+
     def need_tunnel(self):
         if not self._is_listen_localhost():
             return False
-        return self.transport in ["ssh", "ext"]
+        return self.transport == "ssh"
 
-    def is_bad_localhost(self):
-        """
-        Return True if the guest is listening on localhost, but the libvirt
-        URI doesn't give us any way to tunnel the connection
-        """
-        host = self.get_conn_host()[0]
-        if self.need_tunnel():
-            return False
-        return self.transport and self._is_listen_localhost(host)
+    def bad_config(self):
+        if self.transport and self._is_listen_none():
+            return _("Guest is on a remote host, but is only configured "
+                "to allow local file descriptor connections.")
+
+        if self.need_tunnel() and (self.gtlsport and not self.gport):
+            return _("Guest is configured for TLS only which does not "
+                "work over SSH.")
+
+        if (not self.need_tunnel() and
+            self.transport and
+            self._is_listen_localhost(self.get_conn_host()[0])):
+            return _("Guest is on a remote host with transport '%s' "
+                "but is only configured to listen locally. "
+                "To connect remotely you will need to change the guest's "
+                "listen address." % self.transport)
 
     def get_conn_host(self):
-        host = self._connhost
-        port = self._connport
-        tlsport = None
-
-        if not self.need_tunnel():
-            port = self.gport
-            tlsport = self.gtlsport
-            if not self._is_listen_any():
-                host = self.gaddr
+        """
+        Return host/port/tlsport needed for direct spice/vnc connection
+        """
+        host = self.gaddr
+        port = self.gport
+        tlsport = self.gtlsport
+        if self._is_listen_any():
+            host = self._connhost
 
         return host, port, tlsport
 
+    def get_tunnel_host(self):
+        """
+        Physical host name/port needed for ssh tunnel connection
+        """
+        return self._connhost, self._connport
+
     def logstring(self):
         return ("proto=%s trans=%s connhost=%s connuser=%s "
                 "connport=%s gaddr=%s gport=%s gtlsport=%s gsocket=%s" %
                 (self.gtype, self.transport, self._connhost, self.connuser,
                  self._connport, self.gaddr, self.gport, self.gtlsport,
                  self.gsocket))
-    def console_active(self):
-        if self.gsocket:
-            return True
-        if (self.gport in [None, -1] and self.gtlsport in [None, -1]):
-            return False
-        return True
 
 
 class _TunnelScheduler(object):
@@ -112,7 +111,7 @@ class _TunnelScheduler(object):
     """
     def __init__(self):
         self._thread = None
-        self._queue = Queue.Queue()
+        self._queue = queue.Queue()
         self._lock = threading.Lock()
 
     def _handle_queue(self):
@@ -168,13 +167,13 @@ class _Tunnel(object):
         while True:
             try:
                 new = self._errfd.recv(1024)
-            except:
+            except Exception:
                 break
 
             if not new:
                 break
 
-            errout += new
+            errout += new.decode()
 
         return errout
 
@@ -208,7 +207,7 @@ def _make_ssh_command(ginfo):
     if not ginfo.need_tunnel():
         return None
 
-    host, port, ignore = ginfo.get_conn_host()
+    host, port = ginfo.get_tunnel_host()
 
     # Build SSH cmd
     argv = ["ssh", "ssh"]
@@ -249,7 +248,7 @@ def _make_ssh_command(ginfo):
     argv.append("sh -c")
     argv.append("'%s'" % nc_cmd)
 
-    argv_str = reduce(lambda x, y: x + " " + y, argv[1:])
+    argv_str = functools.reduce(lambda x, y: x + " " + y, argv[1:])
     logging.debug("Pre-generated ssh command for ginfo: %s", argv_str)
     return argv
 
@@ -284,10 +283,12 @@ class SSHTunnels(object):
         self.unlock()
 
     def get_err_output(self):
-        errout = ""
+        errstrings = []
         for l in self._tunnels:
-            errout += l.get_err_output()
-        return errout
+            e = l.get_err_output().strip()
+            if e and e not in errstrings:
+                errstrings.append(e)
+        return "\n".join(errstrings)
 
     def _lock(self):
         _tunnel_scheduler.lock()
