@@ -1,22 +1,8 @@
-#
 # Copyright (C) 2009, 2013, 2014 Red Hat, Inc.
 # Copyright (C) 2009 Cole Robinson <crobinso@redhat.com>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-# MA 02110-1301 USA.
-#
+# This work is licensed under the GNU GPLv2 or later.
+# See the COPYING file in the top-level directory.
 
 from gi.repository import GObject
 from gi.repository import Gtk
@@ -35,7 +21,7 @@ def spin_get_helper(widget):
 
     try:
         return int(txt)
-    except:
+    except Exception:
         return adj.get_value()
 
 
@@ -155,14 +141,14 @@ def set_grid_row_visible(child, visible):
     based on UI interraction
     """
     parent = child.get_parent()
-    if type(parent) is not Gtk.Grid:
+    if not isinstance(parent, Gtk.Grid):
         raise RuntimeError("Programming error, parent must be grid, "
                            "not %s" % type(parent))
 
     row = child_get_property(parent, child, "top-attach")
-    for child in parent.get_children():
-        if child_get_property(parent, child, "top-attach") == row:
-            child.set_visible(visible)
+    for c in parent.get_children():
+        if child_get_property(parent, c, "top-attach") == row:
+            c.set_visible(visible)
 
 
 def init_combo_text_column(combo, col):
